package com.rxhttp.compiler

import java.io.BufferedWriter
import java.io.IOException
import javax.annotation.processing.Filer


/**
 * User: ljx
 * Date: 2020/3/31
 * Time: 23:36
 */
object ClassHelper {

    @JvmStatic
    fun generatorStaticClass(filer: Filer, isAndroid: Boolean) {
        generatorBaseRxHttp(filer, isAndroid)
        generatorRxHttpAbstractBodyParam(filer)
        generatorRxHttpBodyParam(filer)
        generatorRxHttpFormParam(filer, isAndroid)
        generatorRxHttpNoBodyParam(filer)
        generatorRxHttpJsonParam(filer)
        generatorRxHttpJsonArrayParam(filer)
        if (isDependenceRxJava()) {
            generatorObservableClass(filer)
        }
    }

    private fun generatorObservableClass(filer: Filer) {
        generatorObservableCall(filer)
        generatorObservableCallEnqueue(filer)
        generatorObservableCallExecute(filer)
        generatorObservableParser(filer)
    }

    private fun generatorBaseRxHttp(filer: Filer, isAndroid: Boolean) {
        if (!isDependenceRxJava()) {
            generatorClass(filer, "BaseRxHttp", """
                package $rxHttpPackage;

                import rxhttp.IRxHttp;

                /**
                 * 本类存放asXxx方法(需要单独依赖RxJava，并告知RxHttp依赖的RxJava版本)
                 * 如未生成，请查看 https://github.com/liujingxing/rxhttp/wiki/FAQ
                 * User: ljx
                 * Date: 2020/4/11
                 * Time: 18:15
                 */
                public abstract class BaseRxHttp implements IRxHttp {

                    
                }
            """.trimIndent())
        } else {
            generatorClass(filer, "BaseRxHttp", """
            package $rxHttpPackage;
            ${
            if (isAndroid) """
            import android.content.Context;
            import android.graphics.Bitmap;
            import android.net.Uri;
            """ else ""
            }
            import java.io.File;
            import java.lang.reflect.Type;
            import java.util.List;
            import java.util.Map;

            import ${getClassPath("Observable")};
            import ${getClassPath("Scheduler")};
            import ${getClassPath("Consumer")};
            import ${getClassPath("RxJavaPlugins")};
            ${if (isAndroid) "import ${getClassPath("Schedulers")};" else ""}
            import okhttp3.Headers;
            import okhttp3.Response;
            import rxhttp.IRxHttp;
            import rxhttp.wrapper.OkHttpCompat;
            import rxhttp.wrapper.callback.OutputStreamFactory;
            ${if (isAndroid) "import rxhttp.wrapper.callback.UriFactory;" else ""}
            import rxhttp.wrapper.entity.ParameterizedTypeImpl;
            import rxhttp.wrapper.entity.Progress;
            ${if (isAndroid) "import rxhttp.wrapper.parse.BitmapParser;" else ""}
            import rxhttp.wrapper.parse.OkResponseParser;
            import rxhttp.wrapper.parse.Parser;
            import rxhttp.wrapper.parse.SimpleParser;
            import rxhttp.wrapper.parse.StreamParser;
            import rxhttp.wrapper.utils.LogUtil;
            ${if (isAndroid) "import rxhttp.wrapper.utils.UriUtil;" else ""}

            /**
             * 本类存放asXxx方法(需要单独依赖RxJava，并告知RxHttp依赖的RxJava版本)
             * 如未生成，请查看 https://github.com/liujingxing/rxhttp/wiki/FAQ
             * User: ljx
             * Date: 2020/4/11
             * Time: 18:15
             */
            public abstract class BaseRxHttp implements IRxHttp {

                static {                   
                    Consumer<? super Throwable> errorHandler = RxJavaPlugins.getErrorHandler();
                    if (errorHandler == null) {                                                
                        /*                                                                     
                        RxJava2的一个重要的设计理念是：不吃掉任何一个异常, 即抛出的异常无人处理，便会导致程序崩溃                      
                        这就会导致一个问题，当RxJava2“downStream”取消订阅后，“upStream”仍有可能抛出异常，                
                        这时由于已经取消订阅，“downStream”无法处理异常，此时的异常无人处理，便会导致程序崩溃                       
                        */                                                                     
                        RxJavaPlugins.setErrorHandler(LogUtil::log);                           
                    }                                                                          
                }                                                                              

                public abstract <T> Observable<T> asParser(Parser<T> parser, Scheduler scheduler, Consumer<Progress> progressConsumer);
                
                public <T> Observable<T> asParser(Parser<T> parser) {
                    return asParser(parser, null, null);
                }

                public final <T> Observable<T> asClass(Class<T> type) {
                    return asParser(new SimpleParser<>(type));
                }

                public final Observable<String> asString() {
                    return asClass(String.class);
                }

                public final Observable<Boolean> asBoolean() {
                    return asClass(Boolean.class);
                }

                public final Observable<Byte> asByte() {
                    return asClass(Byte.class);
                }

                public final Observable<Short> asShort() {
                    return asClass(Short.class);
                }

                public final Observable<Integer> asInteger() {
                    return asClass(Integer.class);
                }

                public final Observable<Long> asLong() {
                    return asClass(Long.class);
                }

                public final Observable<Float> asFloat() {
                    return asClass(Float.class);
                }

                public final Observable<Double> asDouble() {
                    return asClass(Double.class);
                }

                public final <K> Observable<Map<K, K>> asMap(Class<K> kType) {
                    return asMap(kType, kType);
                }

                public final <K, V> Observable<Map<K, V>> asMap(Class<K> kType, Class<V> vType) {
                    Type tTypeMap = ParameterizedTypeImpl.getParameterized(Map.class, kType, vType);
                    return asParser(new SimpleParser<>(tTypeMap));
                }

                public final <T> Observable<List<T>> asList(Class<T> tType) {
                    Type tTypeList = ParameterizedTypeImpl.get(List.class, tType);
                    return asParser(new SimpleParser<>(tTypeList));
                }
                ${
                if (isAndroid) """
                public final <T> Observable<Bitmap> asBitmap() {
                    return asParser(new BitmapParser());
                }
                """ else ""
                }
                public final Observable<Response> asOkResponse() {
                    return asParser(new OkResponseParser());
                }

                public final Observable<Headers> asHeaders() {               
                    return asOkResponse()                                    
                        .map(response -> {                                   
                            try {                                            
                                return response.headers();                   
                            } finally {                                      
                                OkHttpCompat.closeQuietly(response);  
                            }                                                
                        });                                                  
                }

                public final Observable<String> asDownload(String destPath) {
                    return asDownload(destPath, null, null);
                }

                public final Observable<String> asDownload(String destPath,
                                                           Consumer<Progress> progressConsumer) {
                    return asDownload(destPath, null, progressConsumer);
                }
                
                public final Observable<String> asDownload(String destPath, Scheduler scheduler,
                                                           Consumer<Progress> progressConsumer) {
                    return asParser(StreamParser.get(destPath), scheduler, progressConsumer);
                }
                ${
                if (isAndroid) """
                public final Observable<Uri> asDownload(Context context, Uri uri) {
                    return asDownload(context, uri, null, null);   
                }                                                                  
                    
                public final Observable<Uri> asDownload(Context context, Uri uri, Scheduler scheduler,    
                                                           Consumer<Progress> progressConsumer) {            
                    return asParser(StreamParser.get(context, uri), scheduler, progressConsumer);
                }                                                                                            
                """ else ""
                }
                public final <T> Observable<T> asDownload(OutputStreamFactory<T> osFactory) {
                    return asDownload(osFactory, null, null);             
                } 
                                                                                           
                public final <T> Observable<T> asDownload(OutputStreamFactory<T> osFactory, Scheduler scheduler,
                                                           Consumer<Progress> progressConsumer) {
                    return asParser(new StreamParser<>(osFactory), scheduler, progressConsumer);
                }
                
                public final Observable<String> asAppendDownload(String destPath) {                    
                    return asAppendDownload(destPath, null, null);                                     
                }                                                                                      
                                                                                                       
                public final Observable<String> asAppendDownload(String destPath, Scheduler scheduler, 
                                                                 Consumer<Progress> progressConsumer) {
                    long fileLength = new File(destPath).length();                                     
                    setRangeHeader(fileLength, -1, true);                                              
                    return asParser(StreamParser.get(destPath), scheduler, progressConsumer);          
                }                                                                       
                ${
                if (isAndroid) """ 
                public final Observable<Uri> asAppendDownload(Context context, Uri uri) {                   
                    return asAppendDownload(context, uri, null, null);                                      
                }                                                                                           
                                                                                                            
                public final Observable<Uri> asAppendDownload(Context context, Uri uri, Scheduler scheduler,
                                                              Consumer<Progress> progressConsumer) {        
                    return Observable
                        .fromCallable(() -> {
                            long length = UriUtil.length(uri, context);
                            if (length >= 0) setRangeHeader(length, -1, true);
                            return StreamParser.get(context, uri);
                        })
                        .subscribeOn(Schedulers.io())
                        .flatMap(parser -> asParser(parser, scheduler, progressConsumer));        
                }                                                                                           
                    
                public final Observable<Uri> asAppendDownload(UriFactory uriFactory) {                   
                    return asAppendDownload(uriFactory, null, null);                                     
                }                                                                                        
                                                                                                         
                public final Observable<Uri> asAppendDownload(UriFactory uriFactory, Scheduler scheduler,
                                                              Consumer<Progress> progressConsumer) {
                    return Observable
                        .fromCallable(() -> {
                            Uri uri = uriFactory.query();
                            StreamParser<Uri> parser;
                            if (uri != null) {
                                long length = UriUtil.length(uri, uriFactory.getContext());
                                if (length >= 0)
                                    setRangeHeader(length, -1, true);
                                parser = StreamParser.get(uriFactory.getContext(), uri);
                            } else {
                                parser = new StreamParser<>(uriFactory);
                            }
                            return parser;
                        })
                        .subscribeOn(Schedulers.io())
                        .flatMap(parser -> asParser(parser, scheduler, progressConsumer));
                }                                                                                            
                """ else ""
                }    
            }

        """.trimIndent())
        }
    }

    private fun generatorObservableCallEnqueue(filer: Filer) {
        generatorClass(filer, "ObservableCallEnqueue", """
            package $rxHttpPackage;
 
            import java.io.IOException;
            
            import ${getClassPath("Observer")};
            import ${getClassPath("Disposable")};
            import ${getClassPath("Exceptions")};
            import ${getClassPath("RxJavaPlugins")};
            import okhttp3.Call;
            import okhttp3.Callback;
            import okhttp3.Response;
            import rxhttp.IRxHttp;
            import rxhttp.wrapper.callback.ProgressCallback;
            import rxhttp.wrapper.entity.Progress;
            import rxhttp.wrapper.entity.ProgressT;
            import rxhttp.wrapper.utils.LogUtil;

            /**
             * User: ljx
             * Date: 2018/04/20
             * Time: 11:15
             */
            final class ObservableCallEnqueue extends ObservableCall {

                private IRxHttp iRxHttp;
                private boolean callbackUploadProgress;

                ObservableCallEnqueue(IRxHttp iRxHttp) {
                    this(iRxHttp, false);
                }

                ObservableCallEnqueue(IRxHttp iRxHttp, boolean callbackUploadProgress) {
                    this.iRxHttp = iRxHttp;
                    this.callbackUploadProgress = callbackUploadProgress;
                }

                @Override
                public void subscribeActual(Observer<? super Progress> observer) {
                    HttpDisposable d = new HttpDisposable(observer, iRxHttp, callbackUploadProgress);
                    observer.onSubscribe(d);
                    if (d.isDisposed()) {
                        return;
                    }
                    d.run();
                }


                private static class HttpDisposable implements Disposable, Callback, ProgressCallback {

                    private volatile boolean disposed;

                    private final Call call;
                    private final Observer<? super Progress> downstream;

                    /**
                     * Constructs a DeferredScalarDisposable by wrapping the Observer.
                     *
                     * @param downstream the Observer to wrap, not null (not verified)
                     */
                    HttpDisposable(Observer<? super Progress> downstream, IRxHttp iRxHttp, boolean callbackUploadProgress) {
                        if (iRxHttp instanceof RxHttpAbstractBodyParam && callbackUploadProgress) {
                            RxHttpAbstractBodyParam<?, ?> bodyParam = (RxHttpAbstractBodyParam) iRxHttp;
                            bodyParam.getParam().setProgressCallback(this);
                        }
                        this.downstream = downstream;
                        this.call = iRxHttp.newCall();
                    }

                    @Override
                    public void onProgress(Progress p) {
                        if (!disposed) {
                            downstream.onNext(p);
                        }
                    }

                    @Override
                    public void onResponse(Call call, Response response) throws IOException {
                        if (!disposed) {
                            downstream.onNext(new ProgressT<>(response));
                        }
                        if (!disposed) {
                            downstream.onComplete();
                        }
                    }

                    @Override
                    public void onFailure(Call call, IOException e) {
                        LogUtil.log(call.request().url().toString(), e);
                        Exceptions.throwIfFatal(e);
                        if (!disposed) {
                            downstream.onError(e);
                        } else {
                            RxJavaPlugins.onError(e);
                        }
                    }

                    @Override
                    public void dispose() {
                        disposed = true;
                        call.cancel();
                    }

                    @Override
                    public boolean isDisposed() {
                        return disposed;
                    }

                    public void run() {
                        call.enqueue(this);
                    }
                }
            }

        """.trimIndent())
    }

    private fun generatorObservableCallExecute(filer: Filer) {
        generatorClass(filer, "ObservableCallExecute", """
            package $rxHttpPackage;

            import ${getClassPath("Observer")};
            import ${getClassPath("Disposable")};
            import ${getClassPath("Exceptions")};
            import ${getClassPath("RxJavaPlugins")};
            import okhttp3.Call;
            import okhttp3.Response;
            import rxhttp.IRxHttp;
            import rxhttp.wrapper.callback.ProgressCallback;
            import rxhttp.wrapper.entity.Progress;
            import rxhttp.wrapper.entity.ProgressT;
            import rxhttp.wrapper.utils.LogUtil;

            /**
             * User: ljx
             * Date: 2018/04/20
             * Time: 11:15
             */
            final class ObservableCallExecute extends ObservableCall {

                private IRxHttp iRxHttp;
                private boolean callbackUploadProgress;

                ObservableCallExecute(IRxHttp iRxHttp) {
                    this(iRxHttp, false);
                }

                ObservableCallExecute(IRxHttp iRxHttp, boolean callbackUploadProgress) {
                    this.iRxHttp = iRxHttp;
                    this.callbackUploadProgress = callbackUploadProgress;
                }

                @Override
                public void subscribeActual(Observer<? super Progress> observer) {
                    HttpDisposable d = new HttpDisposable(observer, iRxHttp, callbackUploadProgress);
                    observer.onSubscribe(d);
                    if (d.isDisposed()) {
                        return;
                    }
                    d.run();
                }

                private static class HttpDisposable implements Disposable, ProgressCallback {

                    private boolean fusionMode;
                    private volatile boolean disposed;

                    private final Call call;
                    private final Observer<? super Progress> downstream;

                    /**
                     * Constructs a DeferredScalarDisposable by wrapping the Observer.
                     *
                     * @param downstream the Observer to wrap, not null (not verified)
                     */
                    HttpDisposable(Observer<? super Progress> downstream, IRxHttp iRxHttp, boolean callbackUploadProgress) {
                        if (iRxHttp instanceof RxHttpAbstractBodyParam && callbackUploadProgress) {
                            RxHttpAbstractBodyParam<?, ?> bodyParam = (RxHttpAbstractBodyParam) iRxHttp;
                            bodyParam.getParam().setProgressCallback(this);
                        }
                        this.downstream = downstream;
                        this.call = iRxHttp.newCall();
                    }

                    @Override
                    public void onProgress(Progress p) {
                        if (!disposed) {
                            downstream.onNext(p);
                        }
                    }

                    public void run() {
                        Response value;
                        try {
                            value = call.execute();
                        } catch (Throwable e) {
                            LogUtil.log(call.request().url().toString(), e);
                            Exceptions.throwIfFatal(e);
                            if (!disposed) {
                                downstream.onError(e);
                            } else {
                                RxJavaPlugins.onError(e);
                            }
                            return;
                        }
                        if (!disposed) {
                            downstream.onNext(new ProgressT<>(value));
                        }
                        if (!disposed) {
                            downstream.onComplete();
                        }
                    }

                    @Override
                    public void dispose() {
                        disposed = true;
                        call.cancel();
                    }

                    @Override
                    public boolean isDisposed() {
                        return disposed;
                    }
                }
            }

        """.trimIndent())
    }

    private fun generatorObservableCall(filer: Filer) {
        generatorClass(filer, "ObservableCall", """
            package $rxHttpPackage;

            import ${getClassPath("Observable")};
            import ${getClassPath("Scheduler")};
            import ${getClassPath("Consumer")};
            import rxhttp.wrapper.entity.Progress;
            import rxhttp.wrapper.parse.Parser;
            
            /**
             * User: ljx
             * Date: 2020/9/5
             * Time: 21:59
             */
            abstract class ObservableCall extends Observable<Progress> {
            
                public <T> Observable<T> asParser(Parser<T> parser) {
                    return asParser(parser, null, null);
                }
            
                public <T> Observable<T> asParser(Parser<T> parser, Consumer<Progress> progressConsumer) {
                    return asParser(parser, null, progressConsumer);
                }
            
                public <T> Observable<T> asParser(Parser<T> parser, Scheduler scheduler, Consumer<Progress> progressConsumer) {
                    return new ObservableParser<>(this, parser, scheduler, progressConsumer);
                }
            }

        """.trimIndent())
    }

    private fun generatorObservableParser(filer: Filer) {
        generatorClass(filer, "ObservableParser", """
            package $rxHttpPackage;

            import java.util.Objects;
            import java.util.concurrent.atomic.AtomicInteger;

            import ${getClassPath("Observable")};
            import ${getClassPath("ObservableSource")};
            import ${getClassPath("Observer")};
            import ${getClassPath("Exceptions")};
            import ${getClassPath("SimplePlainQueue")};
            import ${getClassPath("SpscArrayQueue")};
            import ${getClassPath("RxJavaPlugins")};
            import ${getClassPath("Scheduler")};
            import ${getClassPath("Disposable")};
            import ${getClassPath("Consumer")};
            import ${getClassPath("DisposableHelper")};
            import ${getClassPath("Scheduler")}.Worker;
            
            import okhttp3.Response;
            import rxhttp.wrapper.annotations.NonNull;
            import rxhttp.wrapper.annotations.Nullable;
            import rxhttp.wrapper.callback.ProgressCallback;
            import rxhttp.wrapper.entity.Progress;
            import rxhttp.wrapper.entity.ProgressT;
            import rxhttp.wrapper.parse.StreamParser;
            import rxhttp.wrapper.parse.Parser;
            import rxhttp.wrapper.utils.LogUtil;

            final class ObservableParser<T> extends Observable<T> {

                private final Parser<T> parser;
                private final ObservableSource<Progress> source;
                private final Scheduler scheduler;
                private final Consumer<Progress> progressConsumer;

                ObservableParser(@NonNull ObservableSource<Progress> source, @NonNull Parser<T> parser,
                                        @Nullable Scheduler scheduler, @Nullable Consumer<Progress> progressConsumer) {
                    this.source = source;
                    this.parser = parser;
                    this.scheduler = scheduler;
                    this.progressConsumer = progressConsumer;
                }

                @Override
                protected void subscribeActual(@NonNull Observer<? super T> observer) {
                    if (scheduler == null) {
                        source.subscribe(new SyncParserObserver<>(observer, parser, progressConsumer));
                    } else {
                        Worker worker = scheduler.createWorker();
                        source.subscribe(new AsyncParserObserver<>(observer, worker, progressConsumer, parser));
                    }
                }

                private static final class SyncParserObserver<T> implements Observer<Progress>, Disposable, ProgressCallback {
                    private final Parser<T> parser;

                    private Disposable upstream;
                    private final Observer<? super T> downstream;
                    private final Consumer<Progress> progressConsumer;
                    private boolean done;

                    SyncParserObserver(Observer<? super T> actual, Parser<T> parser, Consumer<Progress> progressConsumer) {
                        this.downstream = actual;
                        this.parser = parser;
                        this.progressConsumer = progressConsumer;

                        if (progressConsumer != null && parser instanceof StreamParser) {
                            ((StreamParser) parser).setProgressCallback(this);
                        }
                    }

                    @Override
                    public void onSubscribe(Disposable d) {
                        if (DisposableHelper.validate(this.upstream, d)) {
                            this.upstream = d;
                            downstream.onSubscribe(this);
                        }
                    }

                    //download progress callback
                    @Override
                    public void onProgress(Progress p) {
                        if (done) {
                            return;
                        }
                        try {
                            progressConsumer.accept(p);
                        } catch (Throwable t) {
                            fail(t);
                        }
                    }

                    @SuppressWarnings("unchecked")
                    @Override
                    public void onNext(Progress progress) {
                        if (done) {
                            return;
                        }
                        if (progress instanceof ProgressT) {
                            ProgressT<Response> p = (ProgressT<Response>) progress;
                            T v;
                            try {
                                v = Objects.requireNonNull(parser.onParse(p.getResult()), "The onParse function returned a null value.");
                            } catch (Throwable t) {
                                LogUtil.log(p.getResult().request().url().toString(), t);
                                fail(t);
                                return;
                            }
                            downstream.onNext(v);
                        } else {
                            try {
                                progressConsumer.accept(progress);
                            } catch (Throwable t) {
                                fail(t);
                            }
                        }
                    }

                    @Override
                    public void onError(Throwable t) {
                        if (done) {
                            RxJavaPlugins.onError(t);
                            return;
                        }
                        done = true;
                        downstream.onError(t);
                    }

                    @Override
                    public void onComplete() {
                        if (done) {
                            return;
                        }
                        done = true;
                        downstream.onComplete();
                    }

                    @Override
                    public void dispose() {
                        upstream.dispose();
                    }

                    @Override
                    public boolean isDisposed() {
                        return upstream.isDisposed();
                    }

                    private void fail(Throwable t) {
                        Exceptions.throwIfFatal(t);
                        upstream.dispose();
                        onError(t);
                    }
                }


                private static final class AsyncParserObserver<T> extends AtomicInteger
                    implements Observer<Progress>, Disposable, ProgressCallback, Runnable {

                    private final Parser<T> parser;
                    private final Observer<? super T> downstream;

                    private Disposable upstream;
                    private Throwable error;

                    private volatile boolean done;
                    private volatile boolean disposed;
                    private final SimplePlainQueue<Progress> queue;
                    private final Scheduler.Worker worker;

                    private final Consumer<Progress> progressConsumer;

                    AsyncParserObserver(Observer<? super T> actual, Scheduler.Worker worker, Consumer<Progress> progressConsumer, Parser<T> parser) {
                        this.downstream = actual;
                        this.parser = parser;
                        this.worker = worker;
                        this.progressConsumer = progressConsumer;
                        queue = new SpscArrayQueue<>(2);

                        if (progressConsumer != null && parser instanceof StreamParser) {
                            ((StreamParser) parser).setProgressCallback(this);
                        }
                    }

                    @Override
                    public void onSubscribe(@NonNull Disposable d) {
                        if (DisposableHelper.validate(this.upstream, d)) {
                            this.upstream = d;
                            downstream.onSubscribe(this);
                        }
                    }

                    //download progress callback
                    @Override
                    public void onProgress(Progress p) {
                        if (done) {
                            return;
                        }
                        offer(p);
                    }

                    @SuppressWarnings("unchecked")
                    @Override
                    public void onNext(Progress progress) {
                        if (done) {
                            return;
                        }
                        ProgressT<T> pt = null;
                        if (progress instanceof ProgressT) {
                            ProgressT<Response> progressT = (ProgressT<Response>) progress;
                            try {
                                T t = Objects.requireNonNull(parser.onParse(progressT.getResult()), "The onParse function returned a null value.");
                                pt = new ProgressT<>(t);
                            } catch (Throwable t) {
                                LogUtil.log(progressT.getResult().request().url().toString(), t);
                                onError(t);
                                return;
                            }
                        }
                        Progress p = pt != null ? pt : progress;
                        offer(p);
                    }
                    
                    private void offer(Progress p) {
                        if (!queue.offer(p)) {
                            queue.poll();
                            queue.offer(p);
                        }
                        schedule();
                    }

                    @Override
                    public void onError(Throwable t) {
                        if (done) {
                            RxJavaPlugins.onError(t);
                            return;
                        }
                        error = t;
                        done = true;
                        schedule();
                    }

                    @Override
                    public void onComplete() {
                        if (done) {
                            return;
                        }
                        done = true;
                        schedule();
                    }


                    void schedule() {
                        if (getAndIncrement() == 0) {
                            worker.schedule(this);
                        }
                    }

                    @SuppressWarnings("unchecked")
                    @Override
                    public void run() {
                        int missed = 1;

                        final SimplePlainQueue<Progress> q = queue;
                        final Observer<? super T> a = downstream;
                        while (!checkTerminated(done, q.isEmpty(), a)) {
                            for (; ; ) {
                                boolean d = done;
                                Progress p;
                                try {
                                    p = q.poll();

                                    boolean empty = p == null;

                                    if (checkTerminated(d, empty, a)) {
                                        return;
                                    }
                                    if (empty) {
                                        break;
                                    }
                                    if (p instanceof ProgressT) {
                                        a.onNext(((ProgressT<T>) p).getResult());
                                    } else {
                                        progressConsumer.accept(p);
                                    }
                                } catch (Throwable ex) {
                                    Exceptions.throwIfFatal(ex);
                                    disposed = true;
                                    upstream.dispose();
                                    q.clear();
                                    a.onError(ex);
                                    worker.dispose();
                                    return;
                                }
                            }
                            missed = addAndGet(-missed);
                            if (missed == 0) {
                                break;
                            }
                        }
                    }

                    boolean checkTerminated(boolean d, boolean empty, Observer<? super T> a) {
                        if (isDisposed()) {
                            queue.clear();
                            return true;
                        }
                        if (d) {
                            Throwable e = error;
                            if (e != null) {
                                disposed = true;
                                queue.clear();
                                a.onError(e);
                                worker.dispose();
                                return true;
                            } else if (empty) {
                                disposed = true;
                                a.onComplete();
                                worker.dispose();
                                return true;
                            }
                        }
                        return false;
                    }

                    @Override
                    public void dispose() {
                        if (!disposed) {
                            disposed = true;
                            upstream.dispose();
                            worker.dispose();
                            if (getAndIncrement() == 0) {
                                queue.clear();
                            }
                        }
                    }

                    @Override
                    public boolean isDisposed() {
                        return disposed;
                    }
                }
            }

        """.trimIndent())
    }

    private fun generatorRxHttpAbstractBodyParam(filer: Filer) {
        if (!isDependenceRxJava()) {
            generatorClass(filer, "RxHttpAbstractBodyParam", """
                package $rxHttpPackage;
                
                import rxhttp.wrapper.param.AbstractBodyParam;

                /**
                 * Github
                 * https://github.com/liujingxing/rxhttp
                 * https://github.com/liujingxing/rxlife
                 * https://github.com/liujingxing/rxhttp/wiki/FAQ
                 * https://github.com/liujingxing/rxhttp/wiki/更新日志
                 */
                @SuppressWarnings("unchecked")
                public class RxHttpAbstractBodyParam<P extends AbstractBodyParam<P>, R extends RxHttpAbstractBodyParam<P, R>> extends RxHttp<P, R> {

                    protected RxHttpAbstractBodyParam(P param) {
                        super(param);
                    }

                    public final R setUploadMaxLength(long maxLength) {
                        param.setUploadMaxLength(maxLength);
                        return (R) this;
                    }
                }
            """.trimIndent())
        } else {
            generatorClass(filer, "RxHttpAbstractBodyParam", """
                package $rxHttpPackage;
                
                import ${getClassPath("Observable")};
                import ${getClassPath("Scheduler")};
                import ${getClassPath("Consumer")};
                import rxhttp.wrapper.entity.Progress;
                import rxhttp.wrapper.param.AbstractBodyParam;
                import rxhttp.wrapper.parse.Parser;
                
                /**
                 * Github
                 * https://github.com/liujingxing/rxhttp
                 * https://github.com/liujingxing/rxlife
                 * https://github.com/liujingxing/rxhttp/wiki/FAQ
                 * https://github.com/liujingxing/rxhttp/wiki/更新日志
                 */
                @SuppressWarnings("unchecked")
                public class RxHttpAbstractBodyParam<P extends AbstractBodyParam<P>, R extends RxHttpAbstractBodyParam<P, R>> extends RxHttp<P, R> {

                    //Controls the downstream callback thread
                    private Scheduler observeOnScheduler;

                    //Upload progress callback
                    private Consumer<Progress> progressConsumer;

                    protected RxHttpAbstractBodyParam(P param) {
                        super(param);
                    }

                    public final R setUploadMaxLength(long maxLength) {
                        param.setUploadMaxLength(maxLength);
                        return (R) this;
                    }

                    public final R upload(Consumer<Progress> progressConsumer) {
                        return upload(null, progressConsumer);
                    }

                    /**
                     * @param progressConsumer   Upload progress callback
                     * @param observeOnScheduler Controls the downstream callback thread
                     */
                    public final R upload(Scheduler observeOnScheduler, Consumer<Progress> progressConsumer) {
                        this.progressConsumer = progressConsumer;
                        this.observeOnScheduler = observeOnScheduler;
                        return (R) this;
                    }

                    @Override
                    public final <T> Observable<T> asParser(Parser<T> parser) {
                        return asParser(parser, observeOnScheduler, progressConsumer);
                    }

                    @Override
                    public final <T> Observable<T> asParser(Parser<T> parser, Scheduler scheduler,
                                                            Consumer<Progress> progressConsumer) {
                        if (progressConsumer == null) {
                            return super.asParser(parser, scheduler, null);
                        }
                        ObservableCall observableCall;
                        if (isAsync) {
                            observableCall = new ObservableCallEnqueue(this, true);
                        } else {
                            observableCall = new ObservableCallExecute(this, true);
                        }
                        return observableCall.asParser(parser, scheduler, progressConsumer);
                    }
                }

        """.trimIndent())
        }
    }

    private fun generatorRxHttpNoBodyParam(filer: Filer) {
        generatorClass(filer, "RxHttpNoBodyParam", """
            package $rxHttpPackage;

            import java.util.List;
            import java.util.Map;
            
            import rxhttp.wrapper.annotations.NonNull;
            import rxhttp.wrapper.param.NoBodyParam;

            /**
             * Github
             * https://github.com/liujingxing/rxhttp
             * https://github.com/liujingxing/rxlife
             * https://github.com/liujingxing/rxhttp/wiki/FAQ
             * https://github.com/liujingxing/rxhttp/wiki/更新日志
             */
            public class RxHttpNoBodyParam extends RxHttp<NoBodyParam, RxHttpNoBodyParam> {
                public RxHttpNoBodyParam(NoBodyParam param) {
                    super(param);
                }
                
                public RxHttpNoBodyParam add(String key, Object value) {
                    return addQuery(key, value);
                }
                
                public RxHttpNoBodyParam add(String key, Object value, boolean isAdd) {
                    if (isAdd) {
                        addQuery(key, value);
                    }
                    return this;
                }
                
                public RxHttpNoBodyParam addAll(Map<String, ?> map) {
                    return addAllQuery(map);
                }

                public RxHttpNoBodyParam addEncoded(String key, Object value) {
                    return addEncodedQuery(key, value);
                }
                
                public RxHttpNoBodyParam addAllEncoded(@NonNull Map<String, ?> map) {
                    return addAllEncodedQuery(map);
                }

                public RxHttpNoBodyParam set(String key, Object value) {
                    return setQuery(key, value);
                }

                public RxHttpNoBodyParam setEncoded(String key, Object value) {
                    return setEncodedQuery(key, value); 
                }
            }

        """.trimIndent())
    }

    @JvmStatic
    fun generatorRxHttpBodyParam(filer: Filer) {
        generatorClass(filer, "RxHttpBodyParam", """
            package $rxHttpPackage;

            import android.content.Context;
            import android.net.Uri;

            import rxhttp.wrapper.annotations.Nullable;
            import rxhttp.wrapper.param.BodyParam;

            import java.io.File;

            import okhttp3.MediaType;
            import okhttp3.RequestBody;
            import okio.ByteString;

            /**
             * Github
             * https://github.com/liujingxing/rxhttp
             * https://github.com/liujingxing/rxlife
             */
            public class RxHttpBodyParam extends RxHttpAbstractBodyParam<BodyParam, RxHttpBodyParam> {
                public RxHttpBodyParam(BodyParam param) {
                    super(param);
                }
                
                public RxHttpBodyParam setBody(RequestBody requestBody) {
                    param.setBody(requestBody);
                    return this;
                }
                
                public RxHttpBodyParam setBody(String content, @Nullable MediaType mediaType) {
                    param.setBody(content, mediaType);
                    return this;
                }
                
                public RxHttpBodyParam setBody(ByteString content, @Nullable MediaType mediaType) {
                    param.setBody(content, mediaType);
                    return this;
                }
                
                public RxHttpBodyParam setBody(byte[] content, @Nullable MediaType mediaType) {
                    param.setBody(content, mediaType);
                    return this;
                }
                
                public RxHttpBodyParam setBody(byte[] content, @Nullable MediaType mediaType, int offset, int byteCount) {
                    param.setBody(content, mediaType, offset, byteCount);
                    return this;
                }
                
                public RxHttpBodyParam setBody(File file) {
                    param.setBody(file);
                    return this;
                }
                
                public RxHttpBodyParam setBody(File file, @Nullable MediaType mediaType) {
                    param.setBody(file, mediaType);
                    return this;
                }
                
                public RxHttpBodyParam setBody(Uri uri, Context context) {
                    param.setBody(uri, context);
                    return this;
                }
                
                public RxHttpBodyParam setBody(Uri uri, Context context, @Nullable MediaType contentType) {
                    param.setBody(uri, context, contentType);
                    return this;
                }
                
                public RxHttpBodyParam setJsonBody(Object object) {
                    param.setJsonBody(object);
                    return this;
                }
            }

        """.trimIndent())
    }

    private fun generatorRxHttpFormParam(filer: Filer, isAndroid: Boolean) {
        generatorClass(filer, "RxHttpFormParam", """
            package $rxHttpPackage;

            ${if (isAndroid) "import android.content.Context;" else ""}
            ${if (isAndroid) "import android.net.Uri;" else ""}

            import java.io.File;
            import java.util.List;
            import java.util.Map;
            import java.util.Map.Entry;

            import okhttp3.Headers;
            import okhttp3.MediaType;
            import okhttp3.MultipartBody.Part;
            import okhttp3.RequestBody;
            import rxhttp.wrapper.annotations.NonNull;
            import rxhttp.wrapper.annotations.Nullable;
            import rxhttp.wrapper.entity.UpFile;
            import rxhttp.wrapper.param.FormParam;
            import rxhttp.wrapper.utils.UriUtil;

            /**
             * Github
             * https://github.com/liujingxing/rxhttp
             * https://github.com/liujingxing/rxlife
             * https://github.com/liujingxing/rxhttp/wiki/FAQ
             * https://github.com/liujingxing/rxhttp/wiki/更新日志
             */
            public class RxHttpFormParam extends RxHttpAbstractBodyParam<FormParam, RxHttpFormParam> {
                public RxHttpFormParam(FormParam param) {
                    super(param);
                }

                public RxHttpFormParam add(String key, Object value) {
                  param.add(key,value);
                  return this;
                }
                
                public RxHttpFormParam add(String key, Object value, boolean isAdd) {
                  if(isAdd) {
                    param.add(key,value);
                  }
                  return this;
                }
                
                public RxHttpFormParam addAll(Map<String, ?> map) {
                  param.addAll(map);
                  return this;
                }
                
                public RxHttpFormParam addEncoded(String key, Object value) {
                    param.addEncoded(key, value);
                    return this;
                }
                
                public RxHttpFormParam addAllEncoded(@NonNull Map<String, ?> map) {
                    param.addAllEncoded(map);
                    return this;
                }

                public RxHttpFormParam removeAllBody() {
                    param.removeAllBody();
                    return this;
                }

                public RxHttpFormParam removeAllBody(String key) {
                    param.removeAllBody(key);
                    return this;
                }

                public RxHttpFormParam set(String key, Object value) {
                    param.set(key, value);
                    return this;
                }

                public RxHttpFormParam setEncoded(String key, Object value) {
                    param.setEncoded(key, value);
                    return this;
                }

                /**
                 * @deprecated please user {@link #addFile(String, File)} instead
                 */
                @Deprecated
                public RxHttpFormParam add(String key, File file) {
                    param.addFile(key, file);
                    return this;
                }

                public RxHttpFormParam addFile(String key, File file) {
                    param.addFile(key, file);
                    return this;
                }

                public RxHttpFormParam addFile(String key, String filePath) {
                    param.addFile(key, filePath);
                    return this;
                }

                public RxHttpFormParam addFile(String key, String filename, String filePath) {
                    param.addFile(key, filename, filePath);
                    return this;
                }

                public RxHttpFormParam addFile(String key, String filename, File file) {
                    param.addFile(key, filename, file);
                    return this;
                }

                public RxHttpFormParam addFile(UpFile file) {
                    param.addFile(file);
                    return this;
                }

                /**
                 * @deprecated please user {@link #addFiles(List)} instead
                 */
                @Deprecated
                public RxHttpFormParam addFile(List<? extends UpFile> fileList) {
                    return addFiles(fileList);
                }
                
                /**
                 * @deprecated please user {@link #addFiles(String, List)} instead
                 */
                @Deprecated
                public <T> RxHttpFormParam addFile(String key, List<T> fileList) {
                    return addFiles(key, fileList);
                }

                public RxHttpFormParam addFiles(List<? extends UpFile> fileList) {
                    param.addFiles(fileList);
                    return this;
                }
                
                public <T> RxHttpFormParam addFiles(Map<String, T> fileMap) {
                    param.addFiles(fileMap);
                    return this;
                }
                
                public <T> RxHttpFormParam addFiles(String key, List<T> fileList) {
                    param.addFiles(key, fileList);
                    return this;
                }

                public RxHttpFormParam addPart(@Nullable MediaType contentType, byte[] content) {
                    param.addPart(contentType, content);
                    return this;
                }

                public RxHttpFormParam addPart(@Nullable MediaType contentType, byte[] content, int offset,
                                               int byteCount) {
                    param.addPart(contentType, content, offset, byteCount);
                    return this;
                }
                ${
                if (isAndroid) """
                public RxHttpFormParam addPart(Context context, Uri uri) {
                    param.addPart(UriUtil.asRequestBody(uri, context));
                    return this;
                }

                public RxHttpFormParam addPart(Context context, String key, Uri uri) {
                    param.addPart(UriUtil.asPart(uri, context, key));
                    return this;
                }

                public RxHttpFormParam addPart(Context context, String key, String fileName, Uri uri) {
                    param.addPart(UriUtil.asPart(uri, context, key, fileName));
                    return this;
                }

                public RxHttpFormParam addPart(Context context, Uri uri, @Nullable MediaType contentType) {
                    param.addPart(UriUtil.asRequestBody(uri, context, contentType));
                    return this;
                }

                public RxHttpFormParam addPart(Context context, String key, Uri uri,
                                               @Nullable MediaType contentType) {
                    param.addPart(UriUtil.asPart(uri, context, key, null, contentType));
                    return this;
                }

                public RxHttpFormParam addPart(Context context, String key, String filename, Uri uri,
                                               @Nullable MediaType contentType) {
                    param.addPart(UriUtil.asPart(uri, context, key, filename, contentType));
                    return this;
                }

                public RxHttpFormParam addParts(Context context, Map<String, Uri> uriMap) {
                    for (Entry<String, Uri> entry : uriMap.entrySet()) {
                        addPart(context, entry.getKey(), entry.getValue());
                    }
                    return this;
                }

                public RxHttpFormParam addParts(Context context, List<Uri> uris) {
                    for (Uri uri : uris) {
                        addPart(context, uri);
                    }
                    return this;
                }

                public RxHttpFormParam addParts(Context context, String key, List<Uri> uris) {
                    for (Uri uri : uris) {
                        addPart(context, key, uri);
                    }
                    return this;
                }

                public RxHttpFormParam addParts(Context context, List<Uri> uris,
                                                @Nullable MediaType contentType) {
                    for (Uri uri : uris) {
                        addPart(context, uri, contentType);
                    }
                    return this;
                }

                public RxHttpFormParam addParts(Context context, String key, List<Uri> uris,
                                                @Nullable MediaType contentType) {
                    for (Uri uri : uris) {
                        addPart(context, key, uri, contentType);
                    }
                    return this;
                }
                """ else ""
        }
                public RxHttpFormParam addPart(Part part) {
                    param.addPart(part);
                    return this;
                }

                public RxHttpFormParam addPart(RequestBody requestBody) {
                    param.addPart(requestBody);
                    return this;
                }

                public RxHttpFormParam addPart(Headers headers, RequestBody requestBody) {
                    param.addPart(headers, requestBody);
                    return this;
                }

                public RxHttpFormParam addFormDataPart(String key, String fileName, RequestBody requestBody) {
                    param.addFormDataPart(key, fileName, requestBody);
                    return this;
                }

                //Set content-type to multipart/form-data
                public RxHttpFormParam setMultiForm() {
                    param.setMultiForm();
                    return this;
                }
                
                //Set content-type to multipart/mixed
                public RxHttpFormParam setMultiMixed() {
                    param.setMultiMixed();
                    return this;
                }
                
                //Set content-type to multipart/alternative
                public RxHttpFormParam setMultiAlternative() {
                    param.setMultiAlternative();
                    return this;
                }
                
                //Set content-type to multipart/digest
                public RxHttpFormParam setMultiDigest() {
                    param.setMultiDigest();
                    return this;
                }
                
                //Set content-type to multipart/parallel
                public RxHttpFormParam setMultiParallel() {
                    param.setMultiParallel();
                    return this;
                }
                
                //Set the MIME type
                public RxHttpFormParam setMultiType(MediaType multiType) {
                    param.setMultiType(multiType);
                    return this;
                }
            }

        """.trimIndent())
    }

    private fun generatorRxHttpJsonParam(filer: Filer) {
        generatorClass(filer, "RxHttpJsonParam", """
            package $rxHttpPackage;

            import com.google.gson.JsonObject;

            import java.util.Map;
            
            import rxhttp.wrapper.param.JsonParam;
            /**
             * Github
             * https://github.com/liujingxing/rxhttp
             * https://github.com/liujingxing/rxlife
             * https://github.com/liujingxing/rxhttp/wiki/FAQ
             * https://github.com/liujingxing/rxhttp/wiki/更新日志
             */
            public class RxHttpJsonParam extends RxHttpAbstractBodyParam<JsonParam, RxHttpJsonParam> {
                public RxHttpJsonParam(JsonParam param) {
                    super(param);
                }

                public RxHttpJsonParam add(String key, Object value) {
                  param.add(key,value);
                  return this;
                }
                
                public RxHttpJsonParam add(String key, Object value, boolean isAdd) {
                  if(isAdd) {
                    param.add(key,value);
                  }
                  return this;
                }
                
                public RxHttpJsonParam addAll(Map<String, ?> map) {
                  param.addAll(map);
                  return this;
                }
                
                /**
                 * 将Json对象里面的key-value逐一取出，添加到另一个Json对象中，
                 * 输入非Json对象将抛出{@link IllegalStateException}异常
                 */
                public RxHttpJsonParam addAll(String jsonObject) {
                    param.addAll(jsonObject);
                    return this;
                }

                /**
                 * 将Json对象里面的key-value逐一取出，添加到另一个Json对象中
                 */
                public RxHttpJsonParam addAll(JsonObject jsonObject) {
                    param.addAll(jsonObject);
                    return this;
                }

                /**
                 * 添加一个JsonElement对象(Json对象、json数组等)
                 */
                public RxHttpJsonParam addJsonElement(String key, String jsonElement) {
                    param.addJsonElement(key, jsonElement);
                    return this;
                }
            }

        """.trimIndent())
    }

    private fun generatorRxHttpJsonArrayParam(filer: Filer) {
        generatorClass(filer, "RxHttpJsonArrayParam", """
            package $rxHttpPackage;

            import com.google.gson.JsonArray;
            import com.google.gson.JsonObject;

            import java.util.List;
            import java.util.Map;
            
            import rxhttp.wrapper.param.JsonArrayParam;

            /**
             * Github
             * https://github.com/liujingxing/rxhttp
             * https://github.com/liujingxing/rxlife
             * https://github.com/liujingxing/rxhttp/wiki/FAQ
             * https://github.com/liujingxing/rxhttp/wiki/更新日志
             */
            public class RxHttpJsonArrayParam extends RxHttpAbstractBodyParam<JsonArrayParam, RxHttpJsonArrayParam> {
                public RxHttpJsonArrayParam(JsonArrayParam param) {
                    super(param);
                }

                public RxHttpJsonArrayParam add(String key, Object value) {
                  param.add(key,value);
                  return this;
                }
                
                public RxHttpJsonArrayParam add(String key, Object value, boolean isAdd) {
                  if(isAdd) {
                    param.add(key,value);
                  }
                  return this;
                }
                
                public RxHttpJsonArrayParam addAll(Map<String, ?> map) {
                  param.addAll(map);
                  return this;
                }

                public RxHttpJsonArrayParam add(Object object) {
                    param.add(object);
                    return this;
                }

                public RxHttpJsonArrayParam addAll(List<?> list) {
                    param.addAll(list);
                    return this;
                }

                /**
                 * 添加多个对象，将字符串转JsonElement对象,并根据不同类型,执行不同操作,可输入任意非空字符串
                 */
                public RxHttpJsonArrayParam addAll(String jsonElement) {
                    param.addAll(jsonElement);
                    return this;
                }

                public RxHttpJsonArrayParam addAll(JsonArray jsonArray) {
                    param.addAll(jsonArray);
                    return this;
                }

                /**
                 * 将Json对象里面的key-value逐一取出，添加到Json数组中，成为单独的对象
                 */
                public RxHttpJsonArrayParam addAll(JsonObject jsonObject) {
                    param.addAll(jsonObject);
                    return this;
                }

                public RxHttpJsonArrayParam addJsonElement(String jsonElement) {
                    param.addJsonElement(jsonElement);
                    return this;
                }

                /**
                 * 添加一个JsonElement对象(Json对象、json数组等)
                 */
                public RxHttpJsonArrayParam addJsonElement(String key, String jsonElement) {
                    param.addJsonElement(key, jsonElement);
                    return this;
                }
            }

        """.trimIndent())
    }

    private fun generatorObservableHttp(filer: Filer) {
        generatorClass(filer, "ObservableHttp", """
                package $rxHttpPackage;


                import java.io.IOException;
                import java.util.concurrent.Callable;

                import ${getClassPath("Observable")};
                import ${getClassPath("Observer")};
                import ${getClassPath("Exceptions")};
                import ${getClassPath("DeferredScalarDisposable")};
                import ${getClassPath("RxJavaPlugins")};
                import okhttp3.Call;
                import okhttp3.OkHttpClient;
                import okhttp3.Request;
                import okhttp3.Response;
                import rxhttp.HttpSender;
                import rxhttp.RxHttpPlugins;
                import rxhttp.wrapper.annotations.NonNull;
                import rxhttp.wrapper.annotations.Nullable;
                import rxhttp.wrapper.cahce.CacheMode;
                import rxhttp.wrapper.cahce.InternalCache;
                import rxhttp.wrapper.exception.CacheReadFailedException;
                import rxhttp.wrapper.param.Param;
                import rxhttp.wrapper.parse.Parser;
                import rxhttp.wrapper.utils.LogUtil;

                /**
                 * 发送Http请求的观察者，管道中断时，请求还未执行完毕，会将请求cancel
                 * User: ljx
                 * Date: 2018/04/20
                 * Time: 11:15
                 */
                final class ObservableHttp<T> extends Observable<T> implements Callable<T> {
                    private final Param param;
                    private final Parser<T> parser;

                    private Call mCall;
                    private Request request;
                    private InternalCache cache;
                    private OkHttpClient okClient;

                    ObservableHttp(OkHttpClient okClient, @NonNull Param param, @NonNull Parser<T> parser) {
                        this.param = param;
                        this.parser = parser;
                        this.okClient = okClient;
                        cache = RxHttpPlugins.getCache();
                    }

                    @Override
                    public void subscribeActual(Observer<? super T> observer) {
                        HttpDisposable d = new HttpDisposable(observer);
                        observer.onSubscribe(d);
                        if (d.isDisposed()) {
                            return;
                        }
                        T value;
                        try {
                            value = requireNonNull(execute(param), "Callable returned null");
                        } catch (Throwable e) {
                            LogUtil.log(param.getUrl(), e);
                            Exceptions.throwIfFatal(e);
                            if (!d.isDisposed()) {
                                observer.onError(e);
                            } else {
                                RxJavaPlugins.onError(e);
                            }
                            return;
                        }
                        d.complete(value);
                    }

                    @Override
                    public T call() throws Exception {
                        return requireNonNull(execute(param), "The callable returned a null value");
                    }


                    //执行请求
                    private T execute(Param param) throws Exception {
                        if (request == null) { //防止失败重试时，重复构造okhttp3.Request对象
                            request = param.buildRequest();
                        }
                        CacheMode cacheMode = param.getCacheMode();
                        if (cacheModeIs(CacheMode.ONLY_CACHE, CacheMode.READ_CACHE_FAILED_REQUEST_NETWORK)) {
                            //读取缓存
                            Response cacheResponse = getCacheResponse(request, param.getCacheValidTime());
                            if (cacheResponse != null) {
                                return parser.onParse(cacheResponse);
                            }
                            if (cacheModeIs(CacheMode.ONLY_CACHE)) //仅读缓存模式下，缓存读取失败，直接抛出异常
                                throw new CacheReadFailedException("Cache read failed");
                        }
                        Call call = mCall = HttpSender.newCall(okClient, request);
                        Response networkResponse = null;
                        try {
                            networkResponse = call.execute();
                            if (cache != null && cacheMode != CacheMode.ONLY_NETWORK) {
                                //非ONLY_NETWORK模式下,请求成功，写入缓存
                                networkResponse = cache.put(networkResponse, param.getCacheKey());
                            }
                        } catch (Exception e) {
                            if (cacheModeIs(CacheMode.REQUEST_NETWORK_FAILED_READ_CACHE)) {
                                //请求失败，读取缓存
                                networkResponse = getCacheResponse(request, param.getCacheValidTime());
                            }
                            if (networkResponse == null)
                                throw e;
                        }
                        return parser.onParse(networkResponse);
                    }

                    private boolean cacheModeIs(CacheMode... cacheModes) {
                        if (cacheModes == null || cache == null) return false;
                        CacheMode cacheMode = param.getCacheMode();
                        for (CacheMode mode : cacheModes) {
                            if (mode == cacheMode) return true;
                        }
                        return false;
                    }
                    
                    private <T> T requireNonNull(T object, String message) {
                        if (object == null) {
                            throw new NullPointerException(message);
                        }
                        return object;
                    }

                    @Nullable
                    private Response getCacheResponse(Request request, long validTime) throws IOException {
                        if (cache == null) return null;
                        Response cacheResponse = cache.get(request, param.getCacheKey());
                        if (cacheResponse != null) {
                            long receivedTime = cacheResponse.receivedResponseAtMillis();
                            if (validTime != -1 && System.currentTimeMillis() - receivedTime > validTime)
                                return null; //缓存过期，返回null
                            return cacheResponse;
                        }
                        return null;
                    }

                    class HttpDisposable extends DeferredScalarDisposable<T> {

                        /**
                         * Constructs a DeferredScalarDisposable by wrapping the Observer.
                         *
                         * @param downstream the Observer to wrap, not null (not verified)
                         */
                        HttpDisposable(Observer<? super T> downstream) {
                            super(downstream);
                        }

                        @Override
                        public void dispose() {
                            cancelRequest(mCall);
                            super.dispose();
                        }
                    }


                    //关闭请求
                    private void cancelRequest(Call call) {
                        if (call != null && !call.isCanceled())
                            call.cancel();
                    }
                }

            """.trimIndent())
    }

    private fun generatorObservableUpload(filer: Filer) {
        generatorClass(filer, "ObservableUpload", """
                package $rxHttpPackage;

                import java.util.concurrent.atomic.AtomicInteger;
                import java.util.concurrent.atomic.AtomicReference;

                import ${getClassPath("Observable")};
                import ${getClassPath("ObservableEmitter")};
                import ${getClassPath("Observer")};
                import ${getClassPath("Disposable")};
                import ${getClassPath("Exceptions")};
                import ${getClassPath("Cancellable")};
                import ${getClassPath("CancellableDisposable")};
                import ${getClassPath("DisposableHelper")};
                import ${getClassPath("SimpleQueue")};
                import ${getClassPath("SpscLinkedArrayQueue")};
                import ${getClassPath("AtomicThrowable")};
                import ${getClassPath("ExceptionHelper")};
                import ${getClassPath("RxJavaPlugins")};
                
                import okhttp3.Call;
                import okhttp3.OkHttpClient;
                import okhttp3.Request;
                import okhttp3.Response;
                import rxhttp.HttpSender;
                import rxhttp.wrapper.entity.Progress;
                import rxhttp.wrapper.entity.ProgressT;
                import rxhttp.wrapper.param.IFile;
                import rxhttp.wrapper.param.Param;
                import rxhttp.wrapper.parse.Parser;
                import rxhttp.wrapper.utils.LogUtil;

                final class ObservableUpload<T> extends Observable<Progress> {
                    private final Param param;
                    private final Parser<T> parser;

                    private Call mCall;
                    private Request mRequest;
                    private OkHttpClient okClient;

                    ObservableUpload(OkHttpClient okClient, Param param, final Parser<T> parser) {
                        this.param = param;
                        this.parser = parser;
                        this.okClient = okClient;
                    }

                    @Override
                    protected void subscribeActual(Observer<? super Progress> observer) {
                        CreateEmitter<Progress> emitter = new CreateEmitter<Progress>(observer) {
                            @Override
                            public void dispose() {
                                cancelRequest(mCall);
                                super.dispose();
                            }
                        };
                        observer.onSubscribe(emitter);

                        try {
                            ProgressT<T> completeProgress = new ProgressT<>(); //上传完成回调
                            ((IFile) param).setProgressCallback((progress, currentSize, totalSize) -> {
                                //这里最多回调100次,仅在进度有更新时,才会回调
                                Progress p = new Progress(progress, currentSize, totalSize);
                                if (p.isFinish()) {
                                    //上传完成的回调，需要带上请求返回值，故这里先保存进度
                                    completeProgress.set(p);
                                } else {
                                    emitter.onNext(p);
                                }
                            });
                            completeProgress.setResult(execute(param));
                            emitter.onNext(completeProgress); //最后一次回调Http执行结果
                            emitter.onComplete();
                        } catch (Throwable e) {
                            LogUtil.log(param.getUrl(), e);
                            Exceptions.throwIfFatal(e);
                            emitter.onError(e);
                        }
                    }

                    //执行请求
                    private T execute(Param param) throws Exception {
                        if (mRequest == null) { //防止失败重试时，重复构造okhttp3.Request对象
                            mRequest = param.buildRequest();
                        }
                        Call call = mCall = HttpSender.newCall(okClient, mRequest);
                        Response response = call.execute();
                        return parser.onParse(response);
                    }

                    //关闭请求
                    private void cancelRequest(Call call) {
                        if (call != null && !call.isCanceled())
                            call.cancel();
                    }

                    static class CreateEmitter<T>
                        extends AtomicReference<Disposable>
                        implements ObservableEmitter<T>, Disposable {

                        private static final long serialVersionUID = -3434801548987643227L;

                        final Observer<? super T> observer;

                        CreateEmitter(Observer<? super T> observer) {
                            this.observer = observer;
                        }

                        @Override
                        public void onNext(T t) {
                            if (t == null) {
                                onError(new NullPointerException("onNext called with null. Null values are generally not allowed in 2.x operators and sources."));
                                return;
                            }
                            if (!isDisposed()) {
                                observer.onNext(t);
                            }
                        }

                        @Override
                        public void onError(Throwable t) {
                            if (!tryOnError(t)) {
                                RxJavaPlugins.onError(t);
                            }
                        }

                        @Override
                        public boolean tryOnError(Throwable t) {
                            if (t == null) {
                                t = new NullPointerException("onError called with null. Null values are generally not allowed in 2.x operators and sources.");
                            }
                            if (!isDisposed()) {
                                try {
                                    observer.onError(t);
                                } finally {
                                    dispose();
                                }
                                return true;
                            }
                            return false;
                        }

                        @Override
                        public void onComplete() {
                            if (!isDisposed()) {
                                try {
                                    observer.onComplete();
                                } finally {
                                    dispose();
                                }
                            }
                        }

                        @Override
                        public void setDisposable(Disposable d) {
                            DisposableHelper.set(this, d);
                        }

                        @Override
                        public void setCancellable(Cancellable c) {
                            setDisposable(new CancellableDisposable(c));
                        }

                        @Override
                        public ObservableEmitter<T> serialize() {
                            return new SerializedEmitter<T>(this);
                        }

                        @Override
                        public void dispose() {
                            DisposableHelper.dispose(this);
                        }

                        @Override
                        public boolean isDisposed() {
                            return DisposableHelper.isDisposed(get());
                        }

                        @Override
                        public String toString() {
                            return String.format("%s{%s}", getClass().getSimpleName(), super.toString());
                        }
                    }

                    /**
                     * Serializes calls to onNext, onError and onComplete.
                     *
                     * @param <T> the value type
                     */
                    static final class SerializedEmitter<T>
                        extends AtomicInteger
                        implements ObservableEmitter<T> {

                        private static final long serialVersionUID = 4883307006032401862L;

                        final ObservableEmitter<T> emitter;

                        final AtomicThrowable error;

                        final SpscLinkedArrayQueue<T> queue;

                        volatile boolean done;

                        SerializedEmitter(ObservableEmitter<T> emitter) {
                            this.emitter = emitter;
                            this.error = new AtomicThrowable();
                            this.queue = new SpscLinkedArrayQueue<T>(16);
                        }

                        @Override
                        public void onNext(T t) {
                            if (emitter.isDisposed() || done) {
                                return;
                            }
                            if (t == null) {
                                onError(new NullPointerException("onNext called with null. Null values are generally not allowed in 2.x operators and sources."));
                                return;
                            }
                            if (get() == 0 && compareAndSet(0, 1)) {
                                emitter.onNext(t);
                                if (decrementAndGet() == 0) {
                                    return;
                                }
                            } else {
                                SimpleQueue<T> q = queue;
                                synchronized (q) {
                                    q.offer(t);
                                }
                                if (getAndIncrement() != 0) {
                                    return;
                                }
                            }
                            drainLoop();
                        }

                        @Override
                        public void onError(Throwable t) {
                            if (!tryOnError(t)) {
                                RxJavaPlugins.onError(t);
                            }
                        }

                        @Override
                        public boolean tryOnError(Throwable t) {
                            if (emitter.isDisposed() || done) {
                                return false;
                            }
                            if (t == null) {
                                t = new NullPointerException("onError called with null. Null values are generally not allowed in 2.x operators and sources.");
                            }
                            if (ExceptionHelper.addThrowable(error, t)) {
                                done = true;
                                drain();
                                return true;
                            }
                            return false;
                        }

                        @Override
                        public void onComplete() {
                            if (emitter.isDisposed() || done) {
                                return;
                            }
                            done = true;
                            drain();
                        }

                        void drain() {
                            if (getAndIncrement() == 0) {
                                drainLoop();
                            }
                        }

                        void drainLoop() {
                            ObservableEmitter<T> e = emitter;
                            SpscLinkedArrayQueue<T> q = queue;
                            AtomicThrowable error = this.error;
                            int missed = 1;
                            for (; ; ) {

                                for (; ; ) {
                                    if (e.isDisposed()) {
                                        q.clear();
                                        return;
                                    }

                                    if (error.get() != null) {
                                        q.clear();
                                        e.onError(error.terminate());
                                        return;
                                    }

                                    boolean d = done;
                                    T v = q.poll();

                                    boolean empty = v == null;

                                    if (d && empty) {
                                        e.onComplete();
                                        return;
                                    }

                                    if (empty) {
                                        break;
                                    }

                                    e.onNext(v);
                                }

                                missed = addAndGet(-missed);
                                if (missed == 0) {
                                    break;
                                }
                            }
                        }

                        @Override
                        public void setDisposable(Disposable d) {
                            emitter.setDisposable(d);
                        }

                        @Override
                        public void setCancellable(Cancellable c) {
                            emitter.setCancellable(c);
                        }

                        @Override
                        public boolean isDisposed() {
                            return emitter.isDisposed();
                        }

                        @Override
                        public ObservableEmitter<T> serialize() {
                            return this;
                        }

                        @Override
                        public String toString() {
                            return emitter.toString();
                        }
                    }

                }
            """.trimIndent())
    }

    private fun generatorObservableDownload(filer: Filer) {
        generatorClass(filer, "ObservableDownload", """
                package $rxHttpPackage;

                import java.util.concurrent.atomic.AtomicInteger;
                import java.util.concurrent.atomic.AtomicReference;
                
                import ${getClassPath("Observable")};
                import ${getClassPath("ObservableEmitter")};
                import ${getClassPath("Observer")};
                import ${getClassPath("Disposable")};
                import ${getClassPath("Exceptions")};
                import ${getClassPath("Cancellable")};
                import ${getClassPath("CancellableDisposable")};
                import ${getClassPath("DisposableHelper")};
                import ${getClassPath("SimpleQueue")};
                import ${getClassPath("SpscLinkedArrayQueue")};
                import ${getClassPath("AtomicThrowable")};
                import ${getClassPath("ExceptionHelper")};
                import ${getClassPath("RxJavaPlugins")};
                import okhttp3.Call;
                import okhttp3.OkHttpClient;
                import okhttp3.Request;
                import okhttp3.Response;
                import rxhttp.HttpSender;
                import rxhttp.wrapper.annotations.NonNull;
                import rxhttp.wrapper.entity.Progress;
                import rxhttp.wrapper.entity.ProgressT;
                import rxhttp.wrapper.parse.DownloadParser;
                import rxhttp.wrapper.utils.LogUtil;

                final class ObservableDownload extends Observable<Progress> {
                    private final Param param;
                    private final String destPath;
                    private final long offsetSize;

                    private Call mCall;
                    private Request mRequest;
                    private OkHttpClient okClient;

                    private int lastProgress; //上次下载进度

                    ObservableDownload(OkHttpClient okClient, Param param, String destPath, long offsetSize) {
                        this.param = param;
                        this.okClient = okClient;
                        this.destPath = destPath;
                        this.offsetSize = offsetSize;
                    }

                    @Override
                    protected void subscribeActual(Observer<? super Progress> observer) {
                        CreateEmitter<Progress> emitter = new CreateEmitter<Progress>(observer) {
                            @Override
                            public void dispose() {
                                cancelRequest(mCall);
                                super.dispose();
                            }
                        };
                        observer.onSubscribe(emitter);
                    
                        try {
                            ProgressT<String> completeProgress = new ProgressT<>();  //下载完成回调
                            Response response = execute(param);
                            String filePath = new DownloadParser(destPath, (progress, currentSize, totalSize) -> {
                                //这里最多回调100次,仅在进度有更新时,才会回调
                                Progress p = new Progress(progress, currentSize, totalSize);
                                if (offsetSize > 0) {
                                    p.addCurrentSize(offsetSize);
                                    p.addTotalSize(offsetSize);
                                    p.updateProgress();
                                    int currentProgress = p.getProgress();
                                    if (currentProgress <= lastProgress) return;
                                    lastProgress = currentProgress;
                                }
                                if (p.isFinish()) {
                                    //下载完成的回调，需要带上本地存储路径，故这里先保存进度
                                    completeProgress.set(p);
                                } else {
                                    emitter.onNext(p);
                                }
                            }).onParse(response);
                            completeProgress.setResult(filePath);
                            emitter.onNext(completeProgress); //最后一次回调文件下载路径
                            emitter.onComplete();
                        } catch (Throwable e) {
                            LogUtil.log(param.getUrl(), e);
                            Exceptions.throwIfFatal(e);
                            emitter.onError(e);
                        }
                    }
                    
                    private Response execute(@NonNull Param param) throws Exception {
                        if (mRequest == null) { //防止失败重试时，重复构造okhttp3.Request对象
                            mRequest = param.buildRequest();
                        }
                        Call call = mCall = HttpSender.newCall(okClient, mRequest);
                        return call.execute();
                    }

                    //关闭请求
                    private void cancelRequest(Call call) {
                        if (call != null && !call.isCanceled())
                            call.cancel();
                    }

                    static class CreateEmitter<T>
                        extends AtomicReference<Disposable>
                        implements ObservableEmitter<T>, Disposable {

                        private static final long serialVersionUID = -3434801548987643227L;

                        final Observer<? super T> observer;

                        CreateEmitter(Observer<? super T> observer) {
                            this.observer = observer;
                        }

                        @Override
                        public void onNext(T t) {
                            if (t == null) {
                                onError(new NullPointerException("onNext called with null. Null values are generally not allowed in 2.x operators and sources."));
                                return;
                            }
                            if (!isDisposed()) {
                                observer.onNext(t);
                            }
                        }

                        @Override
                        public void onError(Throwable t) {
                            if (!tryOnError(t)) {
                                RxJavaPlugins.onError(t);
                            }
                        }

                        @Override
                        public boolean tryOnError(Throwable t) {
                            if (t == null) {
                                t = new NullPointerException("onError called with null. Null values are generally not allowed in 2.x operators and sources.");
                            }
                            if (!isDisposed()) {
                                try {
                                    observer.onError(t);
                                } finally {
                                    dispose();
                                }
                                return true;
                            }
                            return false;
                        }

                        @Override
                        public void onComplete() {
                            if (!isDisposed()) {
                                try {
                                    observer.onComplete();
                                } finally {
                                    dispose();
                                }
                            }
                        }

                        @Override
                        public void setDisposable(Disposable d) {
                            DisposableHelper.set(this, d);
                        }

                        @Override
                        public void setCancellable(Cancellable c) {
                            setDisposable(new CancellableDisposable(c));
                        }

                        @Override
                        public ObservableEmitter<T> serialize() {
                            return new SerializedEmitter<T>(this);
                        }

                        @Override
                        public void dispose() {
                            DisposableHelper.dispose(this);
                        }

                        @Override
                        public boolean isDisposed() {
                            return DisposableHelper.isDisposed(get());
                        }

                        @Override
                        public String toString() {
                            return String.format("%s{%s}", getClass().getSimpleName(), super.toString());
                        }
                    }

                    /**
                     * Serializes calls to onNext, onError and onComplete.
                     *
                     * @param <T> the value type
                     */
                    static final class SerializedEmitter<T>
                        extends AtomicInteger
                        implements ObservableEmitter<T> {

                        private static final long serialVersionUID = 4883307006032401862L;

                        final ObservableEmitter<T> emitter;

                        final AtomicThrowable error;

                        final SpscLinkedArrayQueue<T> queue;

                        volatile boolean done;

                        SerializedEmitter(ObservableEmitter<T> emitter) {
                            this.emitter = emitter;
                            this.error = new AtomicThrowable();
                            this.queue = new SpscLinkedArrayQueue<T>(16);
                        }

                        @Override
                        public void onNext(T t) {
                            if (emitter.isDisposed() || done) {
                                return;
                            }
                            if (t == null) {
                                onError(new NullPointerException("onNext called with null. Null values are generally not allowed in 2.x operators and sources."));
                                return;
                            }
                            if (get() == 0 && compareAndSet(0, 1)) {
                                emitter.onNext(t);
                                if (decrementAndGet() == 0) {
                                    return;
                                }
                            } else {
                                SimpleQueue<T> q = queue;
                                synchronized (q) {
                                    q.offer(t);
                                }
                                if (getAndIncrement() != 0) {
                                    return;
                                }
                            }
                            drainLoop();
                        }

                        @Override
                        public void onError(Throwable t) {
                            if (!tryOnError(t)) {
                                RxJavaPlugins.onError(t);
                            }
                        }

                        @Override
                        public boolean tryOnError(Throwable t) {
                            if (emitter.isDisposed() || done) {
                                return false;
                            }
                            if (t == null) {
                                t = new NullPointerException("onError called with null. Null values are generally not allowed in 2.x operators and sources.");
                            }
                            if (ExceptionHelper.addThrowable(error, t)) {
                                done = true;
                                drain();
                                return true;
                            }
                            return false;
                        }

                        @Override
                        public void onComplete() {
                            if (emitter.isDisposed() || done) {
                                return;
                            }
                            done = true;
                            drain();
                        }

                        void drain() {
                            if (getAndIncrement() == 0) {
                                drainLoop();
                            }
                        }

                        void drainLoop() {
                            ObservableEmitter<T> e = emitter;
                            SpscLinkedArrayQueue<T> q = queue;
                            AtomicThrowable error = this.error;
                            int missed = 1;
                            for (; ; ) {

                                for (; ; ) {
                                    if (e.isDisposed()) {
                                        q.clear();
                                        return;
                                    }

                                    if (error.get() != null) {
                                        q.clear();
                                        e.onError(error.terminate());
                                        return;
                                    }

                                    boolean d = done;
                                    T v = q.poll();

                                    boolean empty = v == null;

                                    if (d && empty) {
                                        e.onComplete();
                                        return;
                                    }

                                    if (empty) {
                                        break;
                                    }

                                    e.onNext(v);
                                }

                                missed = addAndGet(-missed);
                                if (missed == 0) {
                                    break;
                                }
                            }
                        }

                        @Override
                        public void setDisposable(Disposable d) {
                            emitter.setDisposable(d);
                        }

                        @Override
                        public void setCancellable(Cancellable c) {
                            emitter.setCancellable(c);
                        }

                        @Override
                        public boolean isDisposed() {
                            return emitter.isDisposed();
                        }

                        @Override
                        public ObservableEmitter<T> serialize() {
                            return this;
                        }

                        @Override
                        public String toString() {
                            return emitter.toString();
                        }
                    }

                }

            """.trimIndent())
    }

    private fun generatorClass(filer: Filer, className: String, content: String) {
        var writer: BufferedWriter? = null
        try {
            val sourceFile = filer.createSourceFile("$rxHttpPackage.$className")
            writer = BufferedWriter(sourceFile.openWriter())
            writer.write(content)
        } catch (e: Exception) {

        } finally {
            try {
                writer?.close()
            } catch (e: IOException) {
                //Silent
            }
        }
    }
}